@@ -5,7 +5,7 @@ read.shape <- function(filename=NULL) {
 # Purpose: Read an ESRI shapefile
 # Programmer: Tom Kincaid
 # Date: March 1, 2005
-# Last Revised: March 30, 2007
+# Last Revised: October 8, 2008
 # Description:
 #   This function reads either a single shapefile or multiple shapefiles.  For 
 #   multiple shapefiles, all of the shapefiles must be the same type, i.e., 
@@ -38,6 +38,15 @@ read.shape <- function(filename=NULL) {
 #     SpatialPolygonsDataFrame
 ################################################################################
 
+# If necessary, strip the file extension from the file name
+
+   if(!is.null(filename)) {
+      nc <- nchar(filename)
+      if(substr(filename, nc-3, nc) == ".shp") {
+         filename <- substr(filename, 1, nc-4)
+      }
+   }
+
 # Read the shapefile
 
    sfile <- .Call("readShapeFile", filename)
